package gomatrixplus

import (
	"encoding/json"
	"net/http"

	"github.com/matrix-org/gomatrix"
)

type RespNonce struct {
	Nonce string `json:"nonce"`
}

type ReqSynapseRegister struct {
	Nonce       string `json:"nonce"`
	Username    string `json:"username"`
	DisplayName string `json:"displayname,omitempty"`
	Password    string `json:"password"`
	Admin       bool   `json:"admin"`
	Mac         string `json:"mac"`
}
type RespSynapseRegister struct {
	AccessToken string `json:"access_token"`
	UserId      string `json:"user_id"`
	HomeServer  string `json:"home_server"`
	DeviceId    string `json:"device_id"`
}

func (cli *Client) SharedSecretRegistration(req *ReqSynapseRegister, sharedSecret string) (*RespSynapseRegister, error) {
	// get nonce
	u := cli.BuildBaseURL("/_synapse/admin/v1/register")
	var resp1 *RespNonce
	err := cli.MakeRequest(http.MethodGet, u, nil, &resp1)
	if err != nil {
		return nil, err
	}

	// generate hmac
	macAdmin := "notadmin"
	if req.Admin {
		macAdmin = "admin"
	}
	req.Mac = hmacEncrypt(sharedSecret, resp1.Nonce, req.Username, req.Password, macAdmin)

	// set nonce
	req.Nonce = resp1.Nonce

	resp2 := &RespSynapseRegister{}
	u = cli.BuildBaseURL("/_synapse/admin/v1/register")
	err = cli.MakeRequest(http.MethodPost, u, req, resp2)

	if err != nil {
		httpErr, ok := err.(gomatrix.HTTPError)
		if !ok { // network error
			return nil, err
		}
		if httpErr.Code == 401 {
			// body should be RespUserInteractive, if it isn't, fail with the error
			err = json.Unmarshal(httpErr.Contents, &resp2)
			return resp2, err
		}
	}
	return resp2, err
}
