package gomatrixplus_test

import (
	"fmt"
	"regexp"
	"testing"

	"github.com/jaswdr/faker"
	"github.com/matrix-org/gomatrix"
	"github.com/stretchr/testify/assert"
	"github.com/the-clothing-loop/website/server/pkg/gomatrixplus"
)

func TestSharedSecretRegistration(t *testing.T) {
	re := regexp.MustCompile("[^a-z0-9-]")

	// Set this to the correct values
	matrixURL := "http://localhost:8008"
	matrixSharedSecret := "itsabigsecret"
	matrixAdminUserName := "admin"
	matrixAdminPassword := "itsadmin"

	newUser := re.ReplaceAllString(faker.New().Person().Name(), "")
	newPass := faker.New().Internet().Password()

	// Setup client with admin credentials
	cli, _ := gomatrixplus.NewClient(matrixURL, "", "")
	resp1, err := cli.Login(&gomatrix.ReqLogin{
		Type:     "m.login.password",
		User:     matrixAdminUserName,
		Password: matrixAdminPassword,
	})
	if err != nil {
		panic(err)
	}
	cli.SetCredentials(resp1.UserID, resp1.AccessToken)

	// Run SharedSecretRegistration
	resp, err := cli.SharedSecretRegistration(&gomatrixplus.ReqSynapseRegister{
		Username: newUser,
		Password: newPass,
	}, matrixSharedSecret)

	assert.Nil(t, err)
	assert.NotEmpty(t, resp.AccessToken)
	fmt.Printf("User: %v\n", resp.UserId)
	fmt.Printf("Pass: %v\n", newPass)

	// Uncomment the following line if you want to login with the newly created user
	//panic(resp.UserId)
}
